

# -*- coding: utf-8 -*-
#%%

import numpy as np

def rotation_2d(x_data, y_data, alpha, origin=[0,0]):
    '''
    Rotate a dataset to a newly defined origin with origin as defined in input, and orientation alpha with respect to the x-axis

    Input:
        x_data: array with easting- or x-coordinates of dataset that will be rotated
        y_data: array with northing- or y-coordinates of dataset that will be rotated
        alpha: Orientation of old coordinate system minus orientation new coordinate system
        origin: Easting- or x and Northing or y-coordinate of the origin in the old coordinate system around which will be rotated

    Output:
        x_rot: rotated array of new 'x'-coordinates
        y_rot: rotated array of new 'y'-coordinates

    '''
    # Translate coordinates to local origin and transpose to array with coordinates
    
    x_data = np.array(x_data, dtype=float)
    y_data = np.array(y_data, dtype=float)

    translated_array = np.array([x_data - origin[0], y_data - origin[1]], dtype=float)

    # Build 2D rotation matrix

    R = [[np.cos(alpha), -np.sin(alpha)],
         [np.sin(alpha), np.cos(alpha)]]

    # Apply rotation and remove float errors

    rotated_array = np.matmul(R,translated_array)
    rotated_array[np.abs(rotated_array) < 10**-6] = 0

    # Transpose coordinate sets back and parse output
    
    x_rot = rotated_array[0]
    y_rot = rotated_array[1]

    return x_rot, y_rot

def rotation_3d(x_data, y_data, z_data, alpha=0, beta=0, gamma=0, origin=[0,0,0], order='YPR'):
    '''
    Rotate a dataset 3-dimensionally to a newly defined origin with origin as defined in input

    Input:
        x_data: array with easting- or x-coordinates of dataset that will be rotated
        y_data: array with northing- or y-coordinates of dataset that will be rotated
        z_data: array with elevation- or z-coordinates of dataset that will be rotated
        alpha: Old yaw angle of rotation minus new yaw angle in radians
        beta: Old pitch angle of rotation minus new pitch angle in radians
        gamma: Old roll angle of rotation minus new roll angle in radians
        origin: XYZ coordinates of origin of rotation (single array)
        order: order in which the rotation is applied (YPR means Ry * Rp * Rr)

    Output:
        x_rot: rotated array of new 'x'-coordinates
        y_rot: rotated array of new 'y'-coordinates
        z_rot: rotated array of new 'z'-coordinates

    '''
    x_data = np.array(x_data, dtype=float)
    y_data = np.array(y_data, dtype=float)
    z_data = np.array(z_data, dtype=float)

    translated_array = np.array([x_data - origin[0], y_data - origin[1], z_data  - origin[2]], dtype=float)

    R_yaw = np.array([[np.cos(alpha), -np.sin(alpha), 0],   # Around z-axis
                      [np.sin(alpha), np.cos(alpha) , 0],
                      [0            , 0             , 1]])
    
    R_pitch = np.array([[np.cos(beta) , 0, np.sin(beta)],   # Around y-axis
                        [0            , 1, 0           ],
                        [-np.sin(beta), 0, np.cos(beta)]])

    R_roll = np.array([[1, 0            , 0             ],  # Around x-axis
                       [0, np.cos(gamma), -np.sin(gamma)],
                       [0, np.sin(gamma), np.cos(gamma) ]])

    if order == 'YPR':
        rotated_array = np.matmul(R_yaw, np.matmul(R_pitch, np.matmul(R_roll, translated_array)))
    
    elif order == 'YRP':
        rotated_array = np.matmul(R_yaw, np.matmul(R_roll, np.matmul(R_pitch, translated_array)))

    elif order == 'PYR':
        rotated_array = np.matmul(R_pitch, np.matmul(R_yaw, np.matmul(R_roll, translated_array)))
    
    elif order == 'PRY':
        rotated_array = np.matmul(R_pitch, np.matmul(R_roll, np.matmul(R_yaw, translated_array)))
    
    elif order == 'RYP':
        rotated_array = np.matmul(R_roll, np.matmul(R_yaw, np.matmul(R_pitch, translated_array)))
    
    elif order == 'RPY':
        rotated_array = np.matmul(R_roll, np.matmul(R_pitch, np.matmul(R_yaw, translated_array)))

    else:
        raise ValueError("Insert a string with acombination of Y, P, and R (Yaw, Pitch, and Roll) as eg. 'YPR'")
    
    # Apply rotation and remove float errors

    rotated_array[np.abs(rotated_array) < 10**-6] = 0

    # Transpose coordinate sets back and parse output
    
    x_rot = rotated_array[0]
    y_rot = rotated_array[1]
    z_rot = rotated_array[2]

    return x_rot, y_rot, z_rot
    
# %%    
